/**
 * php var_dump() port to javascript.
 *
 * Original version is here: http://phpjs.org/functions/var_dump.
 * This version is adapted for Plasma JS API, some improvements are made (see comments in function).
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @license MIT (http://opensource.org/licenses/mit-license.php)
 * @author Brett Zamir [http://brett-zamir.me]
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

function var_dump () {
    // http://kevin.vanzonneveld.net
    // +   original by: Brett Zamir (http://brett-zamir.me)
    // +   adapted for Plasma JS API: Nikita Melnichenko (http://nikita.melnichenko.name)
    // +   max depth check: Nikita Melnichenko (http://nikita.melnichenko.name)
    // +   recursion detection by: Nikita Melnichenko (http://nikita.melnichenko.name)
    // *     example 1: var_dump(1);
    // *     returns 1: 'int(1)'

    var output = "", pad_char = " ", pad_val = 4, lgth = 0, i = 0;
    var max_depth = 10;

    var repeat_char = function (len, pad_char) {
        var str = "";
        for (var i=0; i < len; i++) {
            str += pad_char;
        }
        return str;
    };
    var getScalarVal = function (val) {
        var ret = '';
        if (val === null) {
            ret = 'NULL';
        }
        else if (typeof val === 'boolean') {
            ret = 'bool('+val+')';
        }
        else if (typeof val === 'string') {
            ret = 'string('+val.length+') "'+val+'"';
        }
        else if (typeof val === 'number') {
            if (parseFloat(val) == parseInt(val, 10)) {
                ret = 'int('+val+')';
            }
            else {
                ret = 'float('+val+')';
            }
        }
        else if (val === undefined) {
            ret = 'UNDEFINED'; // Not PHP behavior, but neither is undefined as value
        }
        else if (typeof val === 'function') {
            ret = 'FUNCTION'; // Not PHP behavior, but neither is function as value
        }
        return ret;
    };

    var formatArray = function (obj, cur_depth, pad_val, pad_char, stack) {
        var someProp = '';
        if (cur_depth > 0) {
            cur_depth++;
        }
        if (stack !== undefined)
            for (var i in stack) {
                if (stack[i] === obj)
                    return "recursion: "+(stack.length-i)+" level(s) up\n";
            }
        if (cur_depth / 2 > max_depth)
            return "array(...)\n";

        var base_pad = repeat_char(pad_val*(cur_depth-1), pad_char);
        var thick_pad = repeat_char(pad_val*(cur_depth+1), pad_char);
        var str = "";
        var val='';

        if (typeof obj === 'object' && obj !== null) {
            var local_stack = new Array();
            for (var i in stack)
                local_stack[i] = stack[i];
            local_stack.push(obj);
            lgth = 0;
            for (someProp in obj) {
                lgth++;
            }
            str += "array("+lgth+") {\n";
            for (var key in obj) {
                if (typeof obj[key] === 'object' && obj[key] !== null) {
                    str += thick_pad + "["+key+"] =>\n"+thick_pad+formatArray(obj[key], cur_depth+1, pad_val, pad_char, local_stack);
                } else {
                    val = getScalarVal(obj[key]);
                    str += thick_pad + "["+key+"] =>\n"+thick_pad + val + "\n";
                }
            }
            str += base_pad + "}\n";
        } else {
            str = getScalarVal(obj);
        }
        return str;
    };

    output = formatArray(arguments[0], 0, pad_val, pad_char);
    for (i=1; i < arguments.length; i++) {
        output += '\n'+formatArray(arguments[i], 0, pad_val, pad_char);
    }

    print(output);
}
