/**
 * Action for executing on event.
 *
 * Curently supported features:
 * - show notification via KNotify system
 * - execute command
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

plasmoid.include_once("exec.js"); // for knotify, runCommand
plasmoid.include_once("3rdparty/str_replace.js"); // for str_replace

/**
 * Constructor.
 *
 * An action as a list of tasks to execute.
 */
function JAction()
{
	// notification properties
	this.use_knotify = false;
	this.knotify_type = KNotify.TypeNotification;
	this.knotify_template = "";
	this.knotify_timeout = 0; // timeout in seconds

	// command properties
	this.use_command = false;
	this.command_line = "";
}

/**
 * Execute the action.
 *
 * @param object data dialog parameters
 * @return bool true, if script exists
 */
JAction.prototype.execute = function(progress)
{
	$.debug_print("@@@ JAction: execute(" + progress.name + ")");
	if (this.use_knotify)
	{
		var message = str_replace(["%{name}", "%{title}"], [progress.name, progress.title], this.knotify_template);
		$.message(KNotify.Type.Export(this.knotify_type) + ": " + message);
		KNotify(this.knotify_type, message, this.knotify_timeout * 1000);
	}

	if (this.use_command)
	{
		$.message("Executing command: " + this.command_line);
		runCommandLine(this.command_line);
	}
};

/**
 * Export properties for configuration.
 *
 * The first letter is capital because 'export' gives syntax error (bindings issue?).
 *
 * @return object exported item
 */
JAction.prototype.Export = function()
{
	var o = {};
	o.use_knotify = this.use_knotify;
	o.knotify_type = KNotify.Type.Export(this.knotify_type);
	o.knotify_template = this.knotify_template;
	o.knotify_timeout = this.knotify_timeout;
	o.use_command = this.use_command;
	o.command_line = this.command_line;
	return o;
};

/**
 * Import properties after configuration.
 *
 * The first letter is capital because 'import' gives syntax error (bindings issue?).
 *
 * @param object o imported data
 */
JAction.prototype.Import = function(o)
{
	if (!o)
		return;
	this.use_knotify = importBoolean(o.use_knotify, this.use_knotify);
	this.knotify_type = KNotify.Type.Import(o.knotify_type, this.knotify_type);
	this.knotify_template = importString(o.knotify_template, this.knotify_template);
	this.knotify_timeout = importInt(o.knotify_timeout, this.knotify_timeout);
	this.use_command = importBoolean(o.use_command, this.use_command);
	this.command_line = importString(o.command_line, this.command_line);
};
