/**
 * JTimer class for countdown time progress.
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Nikita Melnichenko [http://nikita.melnichenko.name]
 * @copyright Copyright 2010, Nikita Melnichenko
 */

plasmoid.include_once("JLiveProgress.class.js"); // JTimer inherits this

/**
 * Constructor.
 *
 * A countdown timer.
 */
function JTimer()
{
	// parent constructor
	JLiveProgress.call(this);

	this.type = $.ProgressTypeTimer;

	// value, min, max is now milliseconds
	// value is zero by default
	this._value = 0;
	// min is constant, nobody should change it
	this._minimum = 0;
	delete this.minimumChanged;
	// interval is 1 minute by default
	this._maximum = 60000;
	// speed is constant = 1000 milliseconds per second, nobody should change it
	this._speed = 1000;
	delete this.speedChanged;
}

// prototype inheritance
JTimer.inheritsPrototype(JLiveProgress);

/**
 * Property: time.
 *
 * It's alias to value.
 */
JTimer.prototype.__defineGetter__("time", function()
{
	return this.value;
});
JTimer.prototype.__defineSetter__("time", function(value)
{
	this.value = value;
});

/**
 * Property: time interval.
 *
 * It's alias to maximum.
 */
JTimer.prototype.__defineGetter__("interval", function()
{
	return this.maximum;
});
JTimer.prototype.__defineSetter__("interval", function(value)
{
	this.maximum = value;
});

/**
 * Export properties for configuration.
 *
 * The first letter is capital because 'export' gives syntax error (bindings issue?).
 *
 * @return object exported item
 */
JTimer.prototype.Export = function()
{
	var o = JLiveProgress.prototype.Export.call(this);
	// remove some members that shouldn't be exported
	delete o.value;
	delete o.minimum;
	delete o.maximum;
	delete o.speed;
	// export members
	o.type = $.ProgressType.Export(this.type);
	o.time = this.time;
	o.interval = this.interval;
	return o;
};

/**
 * Import properties after configuration.
 *
 * The first letter is capital because 'import' gives syntax error (bindings issue?).
 *
 * @param object o imported data
 */
JTimer.prototype.Import = function(o)
{
	if (!o)
		return;
	this.time = importFloat(o.time, this.time);
	this.interval = importFloat(o.interval, this.interval);

	// remove some members that shouldn't be imported
	var oo = clone(o);
	delete oo.value;
	delete oo.minimum;
	delete oo.maximum;
	delete oo.speed;

	JLiveProgress.prototype.Import.call(this, oo);
};
