#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Configuration dialog for progress properties.
#
# @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
# @author Nikita Melnichenko [http://nikita.melnichenko.name]
# @copyright Copyright 2010, Nikita Melnichenko

import sys
import os
import json
from PyQt4 import QtCore, QtGui, uic

# check number of args
if len(sys.argv) < 2:
	sys.exit(1)

script_dir = os.path.dirname(sys.argv[0])
ui_file = script_dir + '/../ui/dialog-config-progress.ui'

class Dialog():
	def __init__(self, ui_location):
		# create and configure dialog
		self.ui = uic.loadUi(ui_location)
		# dialog icon
		self.ui.setWindowIcon(QtGui.QIcon(script_dir + '/../images/office-chart-pie-16x16.png'))
		# connect std buttons to slot
		self.ui.buttonBox.accepted.connect(self.accept)
		self.ui.buttonBox.rejected.connect(self.reject)
		self.ui.buttonBox.clicked.connect(self.buttonClicked)
		# create and connect delete button
		self.ui.bnDelete = self.ui.buttonBox.addButton("Delete", QtGui.QDialogButtonBox.RejectRole)
		self.ui.bnDelete.setIcon(QtGui.QIcon(script_dir + '/../images/list-remove-16x16.png'))

		# try to load values from input
		try:
			input = json.loads(sys.argv[1])
		except:
			print "Can't parse JSON data"
			sys.exit(3)

		# dialog title
		if 'name' in input:
			self.ui.setWindowTitle(QtCore.QString('Progress %1 properties').arg(input['name']))

		# handler data are needed for accept method
		self.handler = {}
		self.conf = []

		# JProgress title
		self.setupEdit(self.ui.labelTitle, self.ui.editTitle, input, ['title'])
		# JProgress value
		self.setupSpin(self.ui.labelValue, self.ui.dspinValue, input, ['value'])
		# JProgress minimum
		self.setupSpin(self.ui.labelMinimum, self.ui.dspinMinimum, input, ['minimum'])
		# JProgress maximum
		self.setupSpin(self.ui.labelMaximum, self.ui.dspinMaximum, input, ['maximum'])

		# JLiveProgress active
		self.setupCheckBox(self.ui.checkActive, True, input, ['active'])
		# JLiveProgress speed
		self.setupSpin(self.ui.labelSpeed, self.ui.dspinSpeed, input, ['speed'])
		# JLiveProgress tick
		self.setupQTimeEdit(self.ui.labelTick, self.ui.timeTick, input, ['tick'])
		self.ui.timeTick.setMinimumTime(QtCore.QTime(0, 0, 0, 40)) # min is 40 msec = 25 fps

		# JTimer time
		self.setupQTimeEdit(self.ui.labelTime, self.ui.timeTime, input, ['time'])
		# JTimer interval
		self.setupQTimeEdit(self.ui.labelInterval, self.ui.timeInterval, input, ['interval'])

		# JPollProgress command_line
		self.setupEdit(self.ui.labelCommand, self.ui.editCommand, input, ['command_line'])

		if 'ui' in input:
			# JUIProgressCircle color
			self.setupKColorButton(self.ui.labelColor, self.ui.kcbColor, input, ['ui', 'color'])
			# JUIProgressCircle border_color
			self.setupKColorButton(self.ui.labelBorderColor, self.ui.kcbBorderColor, input, ['ui', 'border_color'])
			# JUIProgressCircle background_color
			self.setupKColorButton(self.ui.labelBackgroundColor, self.ui.kcbBackgroundColor, input, ['ui', 'background_color'])
			# JUIProgressCircle background_border_color
			self.setupKColorButton(self.ui.labelBackgroundBorderColor, self.ui.kcbBackgroundBorderColor, input, ['ui', 'background_border_color'])
			# JUIProgressCircle direction
			self.setupCombo(self.ui.labelDirection, self.ui.comboDirection, input, ['ui', 'direction'])
			# JUIProgressCircle starting_point
			self.setupCombo(self.ui.labelStartingPoint, self.ui.comboStartingPoint, input, ['ui', 'starting_point'])
			# JUIProgressCircle displayed_part
			self.setupCombo(self.ui.labelDisplayedPart, self.ui.comboDisplayedPart, input, ['ui', 'displayed_part'])

		if 'actionOnFinish' in input:
			# JAction use_knotify
			self.setupCheckBox(self.ui.groupKNotify, False, input, ['actionOnFinish', 'use_knotify'])
			# JAction knotify_type
			self.setupCombo(self.ui.labelKNotifyType, self.ui.comboKNotifyType, input, ['actionOnFinish', 'knotify_type'])
			# JAction knotify_template
			self.setupEdit(self.ui.labelKNotifyMessage, self.ui.editKNotifyMessage, input, ['actionOnFinish', 'knotify_template'])
			# JAction knotify_timeout
			self.setupSpin(self.ui.labelKNotifyTimeout, self.ui.spinKNotifyTimeout, input, ['actionOnFinish', 'knotify_timeout'])

			# JAction use_command
			self.setupCheckBox(self.ui.groupCommand, False, input, ['actionOnFinish', 'use_command'])
			# JAction command_line
			self.setupEdit(None, self.ui.editActionCommand, input, ['actionOnFinish', 'command_line'])

		# show dialog
		self.ui.show()

	@staticmethod
	def GetValue(input, keys):
		o = input
		for i in keys:
			o = o[i]
		return o

	@staticmethod
	def SetValue(output, keys, value):
		o = output
		l = len(keys)
		for k, v in enumerate(keys):
			if k == l - 1:
				break
			if v not in o:
				o[v] = {}
			o = o[v]
		o[v] = value

	def setupEdit(self, label, edit, input, keys):
		try:
			edit.setText(Dialog.GetValue(input, keys))
			self.conf.append({ 'type': 'Edit', 'widget': edit, 'keys' : keys })
		except:
			if label: label.hide()
			edit.hide()

	def setupSpin(self, label, spin, input, keys):
		try:
			spin.setValue(Dialog.GetValue(input, keys))
			self.conf.append({ 'type': 'Spin', 'widget': spin, 'keys' : keys })
		except:
			if label: label.hide()
			spin.hide()

	def setupCheckBox(self, check, do_hide_on_error, input, keys):
		try:
			check.setChecked(Dialog.GetValue(input, keys))
			self.conf.append({ 'type': 'CheckBox', 'widget': check, 'keys' : keys })
		except:
			check.setChecked(False)
			if do_hide_on_error:
				check.hide()

	def setupCombo(self, label, combo, input, keys):
		try:
			# FIXME: this method would have problems with localization
			index = combo.findText(Dialog.GetValue(input, keys))
			if index >= 0:
				combo.setCurrentIndex(index)
			else:
				raise Exception()
			self.conf.append({ 'type': 'Combo', 'widget': combo, 'keys' : keys })
		except:
			if label: label.hide()
			combo.hide()

	def setupKColorButton(self, label, button, input, keys):
		try:
			color = Dialog.GetValue(input, keys)
			button.setColor(QtGui.QColor(color[0], color[1], color[2]))
			self.conf.append({ 'type': 'KColorButton', 'widget': button, 'keys' : keys })
		except:
			if label: label.hide()
			button.hide()

	def setupQTimeEdit(self, label, spin, input, keys):
		try:
			spin.setTime(QtCore.QTime().addMSecs(Dialog.GetValue(input, keys)))
			self.conf.append({ 'type': 'QTimeEdit', 'widget': spin, 'keys' : keys })
		except:
			if label: label.hide()
			spin.hide()

	# print JSON dump on acceptance
	def accept(self):
		result = {}
		for data in self.conf:
			if data['widget'].isEnabled():
				if data['type'] == 'Edit':
					Dialog.SetValue(result, data['keys'], str(data['widget'].text()))
				if data['type'] == 'Spin':
					Dialog.SetValue(result, data['keys'], data['widget'].value())
				if data['type'] == 'CheckBox':
					Dialog.SetValue(result, data['keys'], data['widget'].isChecked())
				if data['type'] == 'Combo':
					Dialog.SetValue(result, data['keys'], str(data['widget'].currentText()))
				if data['type'] == 'KColorButton':
					qcolor = data['widget'].color()
					Dialog.SetValue(result, data['keys'], [ qcolor.red(), qcolor.green(), qcolor.blue() ])
				if data['type'] == 'QTimeEdit':
					time = data['widget'].time()
					Dialog.SetValue(result, data['keys'], QtCore.QTime().msecsTo(time))

		print json.dumps(result)
		sys.exit(0)

	# exit on rejection
	def buttonClicked(self, button):
		if self.ui.bnDelete == button:
			print "delete"
			sys.exit(100)

	# exit on rejection
	def reject(self):
		print "reject"
		sys.exit(2)


# start application
app = QtGui.QApplication(sys.argv)
dialog = Dialog(ui_file)
app.exec_()
# if we came here this means error cause normal flow should be 'accept' or 'reject' function
print "close"
sys.exit(4)
